"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = _interopRequireWildcard(require("react"));
var _reactNative = require("react-native");
var _VisionSdkViewManager = require("./VisionSdkViewManager");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
var ScanMode;
(function (ScanMode) {
  ScanMode["OCR"] = "ocr";
  ScanMode["BARCODE"] = "barcode";
  ScanMode["QRCODE"] = "qrcode";
})(ScanMode || (ScanMode = {}));
const Camera = _ref => {
  let {
    children,
    refProp,
    reRender,
    captureMode,
    BarCodeScanHandler = _e => {},
    OCRScanHandler = _e => {},
    OnDetectedHandler = _e => {},
    onError = _e => {}
  } = _ref;
  const defaultScanMode = ScanMode.BARCODE;
  const [mode, setMode] = (0, _react.useState)(defaultScanMode);
  const [token, setToken] = (0, _react.useState)('');
  const [apiKey, setApiKey] = (0, _react.useState)('');
  const [environment, setEnvironment] = (0, _react.useState)('dev');
  const [locationId, setLocationId] = (0, _react.useState)('');
  const [options, setOptions] = (0, _react.useState)({
    parse_addresses: 'true',
    match_contacts: 'true'
  });
  const VisionSDKViewRef = (0, _react.useRef)(null);
  (0, _react.useImperativeHandle)(refProp, () => ({
    cameraCaptureHandler: () => {
      onPressCaptures();
    },
    stopRunningHandler: () => {
      onPressStopRunning();
    },
    startRunningHandler: () => {
      onPressStartRunning();
    },
    changeModeHandler: (input, token, locationId, option, appEnvironment) => {
      setEnvironment(appEnvironment ? appEnvironment : environment);
      setToken(token);
      setLocationId(locationId);
      setApiKey(apiKey);
      onChangeMode(input);
      onChangeOptions(option ? option : options);
    }
  }));
  const onPressCaptures = () => {
    _reactNative.UIManager.dispatchViewManagerCommand((0, _reactNative.findNodeHandle)(VisionSDKViewRef.current), _reactNative.UIManager.hasViewManagerConfig('VisionSDKView') && _reactNative.UIManager.getViewManagerConfig('VisionSDKView').Commands.captureImage || 0, []);
  };
  const onPressStopRunning = () => {
    _reactNative.UIManager.dispatchViewManagerCommand((0, _reactNative.findNodeHandle)(VisionSDKViewRef.current), _reactNative.UIManager.hasViewManagerConfig('VisionSDKView') && _reactNative.UIManager.getViewManagerConfig('VisionSDKView').Commands.stopRunning || 1, []);
  };
  const onPressStartRunning = () => {
    _reactNative.UIManager.dispatchViewManagerCommand((0, _reactNative.findNodeHandle)(VisionSDKViewRef.current), _reactNative.UIManager.hasViewManagerConfig('VisionSDKView') && _reactNative.UIManager.getViewManagerConfig('VisionSDKView').Commands.startRunning || 2, []);
  };
  const onChangeMode = input => {
    setMode(input);
  };
  const onChangeOptions = input => {
    setOptions(input);
  };
  (0, _react.useEffect)(() => {
    _reactNative.DeviceEventEmitter.addListener('onBarcodeScanSuccess', BarCodeScanHandler);
    _reactNative.DeviceEventEmitter.addListener('onOCRDataReceived', OCRScanHandler);
    _reactNative.DeviceEventEmitter.addListener('onDetected', OnDetectedHandler);
    return () => {
      _reactNative.DeviceEventEmitter.removeAllListeners('onBarcodeScanSuccess');
      _reactNative.DeviceEventEmitter.removeAllListeners('onOCRDataReceived');
      _reactNative.DeviceEventEmitter.removeAllListeners('onDetected');
    };
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  return /*#__PURE__*/_react.default.createElement(_react.default.Fragment, null, /*#__PURE__*/_react.default.createElement(_VisionSdkViewManager.VisionSdkView, {
    key: reRender,
    style: styles.flex,
    onBarcodeScanSuccess: BarCodeScanHandler,
    onOCRDataReceived: OCRScanHandler,
    onDetected: OnDetectedHandler,
    mode: mode,
    captureMode: captureMode,
    onError: onError,
    token: token,
    locationId: locationId,
    options: _reactNative.Platform.OS === 'ios' ? options : JSON.stringify(options) // ideally this should be passed from options variable, that is receiving data from ScannerContainer
    ,
    environment: environment,
    ref: VisionSDKViewRef
  }, children));
};
var _default = Camera;
exports.default = _default;
const styles = _reactNative.StyleSheet.create({
  flex: {
    flex: 1
  },
  row: {
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'space-between'
  },
  buttonText: {
    fontSize: 15,
    color: 'yellow'
  },
  unbuttonText: {
    fontSize: 15,
    color: 'white'
  },
  zoomBlock: {
    paddingHorizontal: 5,
    borderRadius: 30,
    height: 40,
    flexDirection: 'row',
    backgroundColor: '#00000050',
    justifyContent: 'space-between'
  },
  iconStyle: {
    width: 55,
    height: 55,
    bottom: 8
  },
  autoManualBlock: {
    borderRadius: 4,
    paddingHorizontal: 2,
    height: 40,
    alignItems: 'center',
    flexDirection: 'row',
    backgroundColor: 'grey',
    justifyContent: 'space-between'
  },
  autoManualButton: {
    backgroundColor: '#fff',
    width: 80,
    borderRadius: 4,
    height: 30,
    marginHorizontal: 4,
    paddingHorizontal: 10,
    alignItems: 'center',
    justifyContent: 'center'
  },
  unautoManualButton: {
    backgroundColor: 'grey',
    width: 80,
    borderRadius: 4,
    height: 30,
    marginHorizontal: 4,
    paddingHorizontal: 10,
    alignItems: 'center',
    justifyContent: 'center'
  },
  circle: {
    width: 30,
    height: 30,
    borderRadius: 40,
    marginVertical: 5,
    marginHorizontal: 5,
    backgroundColor: '#00000050'
  },
  childrenContainer: {
    paddingTop: 20,
    justifyContent: 'space-between',
    position: 'absolute',
    top: 0,
    left: 0,
    height: '100%',
    width: '100%'
  }
});
//# sourceMappingURL=index.js.map